/**************************************************************************
 *
 *  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 *  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
 *  PURPOSE.
 *
 *  Copyright (c) 1992-1994 David Johnston.  All Rights Reserved.
 * 
 **************************************************************************/
#include <windows.h>
#include <math.h>        
//#include <stdio.h>

#include "xfms99dll.c" 
#include "resource.h"
#include "commctrl.h"


#pragma pack(1)

// Usa an Internal Structure for your function, and access it through a handle
typedef struct distort_tag
{	WORD		hGraph;					/* Graphical input device */
	HANDLE		hRunTime;
	BOOL		bSymmetric;
	WORD		hGraph2;
	WORD		hGraphViewing;	   // specified as "WORD" in struct since it's an alias
	short		iGraphViewing;
	BOOL		bSplines1;
	BOOL		bSplines2;
} DISTORT;

#pragma pack()
// Structure Definition for Preset and Script handling
// list variable types as:  c char
//							b BOOL
//							i short or WORD 
//							f float
//							l long or DWORD
//							d double
//							g GRAPH
//							h Global Handle  
//							s String of 256 chars
//
// Structure Preset Vars: 'y' to include in presets, 'n' to ignore.

#define STRUCTDEFINITION "ghbgiibb"
#define PRESETDEFINITION "ynyynnyy"
#define TRANSFORMNAME "SDK Distortion"
#define TRANSFORMNAME2 "Distortion2" // for use with presets
#define TRANSFORMGROUP "Special"

//#define dsetitem(Id,dVal) sprintf(tmp,"%g",dVal); SetDlgItemText(hWndDlg,Id,tmp)

#define MEMSIZE 24000

BOOL WINAPI DllMain (HANDLE hModule, DWORD fdwReason, LPVOID lpReserved)
{
   switch (fdwReason)
   {
      case DLL_PROCESS_ATTACH:
        /* Code from LibMain inserted here.  Return TRUE to keep the
            DLL loaded or return FALSE to fail loading the DLL.
 
            You may have to modify the code in your original LibMain to
            account for the fact that it may be called more than once.
            You will get one DLL_PROCESS_ATTACH for each process that
            loads the DLL. This is different from LibMain which gets
            called only once when the DLL is loaded. The only time this
            is critical is when you are using shared data sections.
            If you are using shared data sections for statically
            allocated data, you will need to be careful to initialize it
            only once. Check your code carefully.
 
            Certain one-time initializations may now need to be done for
            each process that attaches. You may also not need code from
            your original LibMain because the operating system may now
            be doing it for you.
         */
         break;
 
      case DLL_THREAD_ATTACH:
         /* Called each time a thread is created in a process that has
            already loaded (attached to) this DLL. Does not get called
            for each thread that exists in the process before it loaded
            the DLL.
 
            Do thread-specific initialization here.
         */
         break;
 
      case DLL_THREAD_DETACH:
         /* Same as above, but called when a thread in the process
            exits.
 
            Do thread-specific cleanup here.
         */
         break;
 
      case DLL_PROCESS_DETACH:
         /* Code from _WEP inserted here.  This code may (like the
            LibMain) not be necessary.  Check to make certain that the
            operating system is not doing it for you.
         */
         break;
   }
 
   /* The return value is only used for DLL_PROCESS_ATTACH; all other
      conditions are ignored.  */
   return TRUE;   // successful DLL_PROCESS_ATTACH
}
// Fill XFMQUERY structure with information regarding this file filter

__declspec(dllexport) short FAR PASCAL QueryXfm(XFMQUERY * cq)
{   lstrcpy(cq->szName,TRANSFORMGROUP);
	if (*cq->szName)
		lstrcpy(cq->szName+lstrlen(cq->szName),"\\");
	lstrcpy(cq->szName+lstrlen(cq->szName),TRANSFORMNAME);

	lstrcpy(cq->szCopyright,"(c)1994-1996 Syntrillium Software Corporation"); 

	cq->wSupports=XFM_MONO16|XFM_STEREO16;
	cq->dwFlags=XF_TRANSFORM|XF_USESPRESETSAPI|XF_USESGRAPHAPI|XF_MUSTHIGHLIGHT|
				XF_USES32BITAPI|XF_AUTOCROSSFADE;
	cq->dwUserDataLength = sizeof(DISTORT);
	lstrcpy(cq->szStructDef,STRUCTDEFINITION);
	lstrcpy(cq->szPresetDef,PRESETDEFINITION);
	lstrcpy(cq->szToolHelp,"Add Distortion/Grunge/Overdrive effects");
	return XFM_VALIDLIBRARY;
} 


__declspec(dllexport) BOOL FAR PASCAL XfmInit(COOLINFO *ci)
{	DISTORT * lpAmp;
	HANDLE hDistort;
	hDistort=ci->hUserData;         
	lpAmp=(DISTORT *)GlobalLock(hDistort);
	if (!lpAmp)
		return FALSE;
	lpAmp->hGraph=GraphCreate(ci,0,4096,0,4096,0,4096); 
	lpAmp->hRunTime=NULL;
	lpAmp->bSymmetric=TRUE;
	lpAmp->hGraph2=GraphCreate(ci,0,4096,0,4096,0,4096);
	lpAmp->hGraphViewing=0;
	lpAmp->iGraphViewing=0; // viewing symmetric 'positive' graph
	lpAmp->bSplines1=FALSE;
	lpAmp->bSplines2=FALSE;
	GlobalUnlock(hDistort);
	return TRUE;
}

__declspec(dllexport) BOOL FAR PASCAL XfmDestroy(COOLINFO *ci)
{   DISTORT * lpAmp;
	HANDLE hDistort;
	hDistort=ci->hUserData;         
	lpAmp=(DISTORT *)GlobalLock(hDistort);
	if (!lpAmp)
		return FALSE;
 	if (lpAmp->hGraph)
 	{	GraphDestroy(ci,lpAmp->hGraph);
 		lpAmp->hGraph=(WORD)-1;
 	}                   
 	if (lpAmp->hGraph2)
 	{	GraphDestroy(ci,lpAmp->hGraph2);
 		lpAmp->hGraph2=(WORD)-1;
 	}                   
 	GlobalUnlock(hDistort); 
	return TRUE;
}

__declspec(dllexport) BOOL FAR PASCAL XfmSetup(HWND hWnd, HINSTANCE hInst, COOLINFO *ci)
{	short nRc;
	FARPROC lpfnDIALOGMsgProc;
	
	lpfnDIALOGMsgProc = GetProcAddress(hInst,(LPCSTR)MAKELONG(100,0));
	
	nRc = DialogBoxParam((HINSTANCE)hInst,(LPCSTR)MAKEINTRESOURCE(IDD_TRANSFORM), (HWND)hWnd, (DLGPROC)lpfnDIALOGMsgProc,(DWORD)ci);
	
	return nRc;
}

_declspec(dllexport) BOOL FAR PASCAL XfmUpdate(COOLINFO *ciSource, COOLINFO * ciDest, DWORD dwCursor)
{
	DISTORT * lpAmpSource=GlobalLock(ciSource->hUserData);
	DISTORT * lpAmpDest=GlobalLock(ciDest->hUserData);
	short * pInts;

	//short i;
	DWORD dwLength=ciDest->dwHiSample-ciDest->dwLoSample+1;

	if (lpAmpDest->hRunTime)
	{
		pInts=(short *)GlobalLock(lpAmpDest->hRunTime);
 	 	GraphCopy(ciDest,lpAmpDest->hGraph,lpAmpSource->hGraph);
		GraphCopy(ciDest,lpAmpDest->hGraph2,lpAmpSource->hGraph2);
		lpAmpDest->bSymmetric=lpAmpSource->bSymmetric;
		pInts[16389]=0;
		
		//for (i=0;i<16385;i++)
		//{	
		//	pInts[i]=min(16383,(short)GraphGetValueAt(ciSource,lpAmpSource->hGraph,(double)i/16384.0));
 		//} 		
		GlobalUnlock(lpAmpDest->hRunTime);
	}

	GlobalUnlock(ciDest->hUserData);
	GlobalUnlock(ciSource->hUserData);

	return TRUE;
}



__declspec(dllexport) BOOL FAR PASCAL XfmDo(COOLINFO *ci)
{	DISTORT * lpAmp = NULL;
	HANDLE hDistort = NULL;
	DWORD losamp, hisamp;
	
	HANDLE hBigMem=NULL;
	unsigned char *data;
	short *datai;
	DWORD length;
	DWORD offset;
	DWORD origsize;                
	DWORD maxsize;
	WORD wAlign;
	WORD wFlags=RWEX_16BIT;
	HANDLE hLogs=NULL;
	double * pLogs=NULL;
	
	//HANDLE hInts=NULL;
	short *pInts=NULL;   
	short i;
	
    
    hDistort=ci->hUserData;         
	lpAmp=(DISTORT *)GlobalLock(hDistort);
	losamp=ci->dwLoSample;
	hisamp=ci->dwHiSample;
	
	wFlags|=(WORD)((ci->wChannels==2)?RWEX_STEREO:RWEX_MONO);
	wAlign=sizeof(short)*ci->wChannels;

	lpAmp->hRunTime=GlobalAlloc(GMEM_MOVEABLE,16390*sizeof(short)*2);
    if (!lpAmp->hRunTime) return 1;
    pInts=(short *)GlobalLock(lpAmp->hRunTime);
    if (!pInts)
    {	GlobalFree(lpAmp->hRunTime);
    	GlobalUnlock(hDistort);
		return 1;
    }
	for (i=0;i<20;i++)
		pInts[i]=i;

	for (i=0;i<20;i++)
		pInts[i+16390]=i;

	maxsize=MEMSIZE/4;
 	maxsize=(maxsize/16)*16;
    
	hBigMem=GlobalAlloc(GMEM_MOVEABLE,(DWORD)MEMSIZE+1000);
	if (hBigMem==NULL)
	{	GlobalUnlock(hDistort);
		return 1;
	}
  
 	data=GlobalLock(hBigMem);
	if (data==NULL) 
	{	GlobalFree(hBigMem);
		GlobalUnlock(hDistort);
		hBigMem=0;
		return 1;
	}
	datai=(short *)data;

	hLogs=GlobalAlloc(GMEM_MOVEABLE,sizeof(double)*16385);
	pLogs=(double *)GlobalLock(hLogs);
	for (i=17;i<=16384;i++)
	{	pLogs[i]=(log10((double)(i)/(16384.0))*20.0+60.0)/60.0;
		if (pLogs[i]<0)
			pLogs[i]=0;
	}
    
    ProgressCreate(ci,"Distorting Selection",NULL);
    
    origsize=length=(hisamp-losamp+1)*wAlign;
 	offset=losamp*wAlign;
 	
	// build short conversion table
	if (lpAmp->bSymmetric)
	{	double fAmpOut;
		for (i=17;i<16385;i++)
		{	fAmpOut=GraphGetValueAt(ci,lpAmp->hGraph,pLogs[i])/4096;
			fAmpOut=pow(10.0,(fAmpOut*60.0-60.0)/20.0);
			pInts[i]=(short)(fAmpOut*16384.0+0.5);
		}
		CopyMemory(pInts+16390,pInts,sizeof(short)*16390);
	}
	else
	{	double fAmpOut;
		for (i=17;i<16385;i++)
		{	fAmpOut=GraphGetValueAt(ci,lpAmp->hGraph,pLogs[i])/4096;
			fAmpOut=pow(10.0,(fAmpOut*60.0-60.0)/20.0);
			pInts[i]=(short)(fAmpOut*16384.0+0.5);
			fAmpOut=GraphGetValueAt(ci,lpAmp->hGraph2,pLogs[i])/4096;
			fAmpOut=pow(10.0,(fAmpOut*60.0-60.0)/20.0);
			pInts[i+16390]=(short)(fAmpOut*16384.0+0.5);		
			//pInts[i]=(short)GraphGetValueAt(ci,lpAmp->hGraph,(double)i/16386.0);
			//pInts[i+16390]=(short)GraphGetValueAt(ci,lpAmp->hGraph2,(double)i/16386.0);			
		}		
	}
	pInts[16389]=16385;  // this is the last mapping calculated successfully	

	while (length!=0)
 	{	DWORD amount,amount2;
 	 	DWORD t;
		short value;
 		short m2;
 		
		amount=length;
		if (amount>maxsize) amount=maxsize;
		amount2=amount/2;
		length-=amount;
		ReadDataEx(ci,data,offset/wAlign,amount/wAlign,wFlags);
		
		if (ci->wChannels==1)
 	 	{	//if (ci->wBitsPerSample==16)
 	 		{	for (t=0;t<amount2;t++)
 	         	{	value=datai[t];
 	         	    if (value<0) // use other conversion table
 	         	    {	value=-value; 	         	    	
						m2=value&1;
 	         			value=(short)(((pInts[16390+(value>>1)])<<1)+m2);
 	         			datai[t]=-value; 	         			
 	         	    }
					else
					{	m2=value&1;
 	         			value=(short)(((pInts[value>>1])<<1)+m2);
 	         			datai[t]=value; 	         		
					}
 	         	}
 	        }
 	    }
 	 	else
 	 	{	//if (ci->wBitsPerSample==16)   // 16-bit stereo
 	 		{	for (t=0;t<amount2;)
 	    	 	{	
 	    	 		value=datai[t];
 	         		if (value<0)
 	         	    {	value=-value; 	         	    	
						m2=value&1; 	         		
 	         			value=(short)(((pInts[16390+(value>>1)])<<1)+m2);
 	         			datai[t]=-value; 	         			
					}
					else
					{	m2=value&1; 	         		
 	         			value=(short)(((pInts[value>>1])<<1)+m2);
 	         			datai[t]=value; 	         			
					}
 	         	    
 	    	 		t++;
 	    	 	 	
 	    	 	 	value=datai[t];
 	         	    if (value<0)
 	         	    {	value=-value; 	         	    	
 	         			m2=value&1; 	         		
 	         			value=(short)(((pInts[16390+(value>>1)])<<1)+m2);
 	         			datai[t]=-value; 	         			
					}                
 	         	    else
					{	m2=value&1; 	         		
 	         			value=(short)(((pInts[value>>1])<<1)+m2);
 	         			datai[t]=value;
					}
 	    	 		t++;
 	    	 	}
 	        }
 	    }           
 	    
 	 	WriteDataEx(ci,data,offset/wAlign,amount/wAlign,wFlags);
 	 	offset+=amount;
 	 	ProgressMeter(ci,(origsize-length),origsize);
		if (*(ci->lpProgressCanceled)) 
			break;
 	 
		if (pInts[16389]<16385)
		{	int iStart=pInts[16389];
			int iEnd=iStart+512;
			if (iEnd>16385)
				iEnd=16385;
			
			pInts[16389]=iEnd;
			
			if (lpAmp->bSymmetric)
			{	double fAmpOut;
				if (iStart<17)
					iStart=17;
				for (i=iStart;i<iEnd;i++)
				{	fAmpOut=GraphGetValueAt(ci,lpAmp->hGraph,pLogs[i])/4096.0;
					fAmpOut=pow(10.0,(fAmpOut*60.0-60.0)/20.0);
					pInts[i]=(short)(fAmpOut*16384.0+0.5);
					//pInts[i]=(short)GraphGetValueAt(ci,lpAmp->hGraph,(double)i/16386.0);
				}
				//pInts[16384]=32765;		
				//pInts[16383]=32765;		
				CopyMemory(pInts+16390+iStart,pInts+iStart,sizeof(short)*(iEnd-iStart));
			}
			else
			{	double fAmpOut;
				if (iStart<17)
					iStart=17;
				for (i=iStart;i<iEnd;i++)
				{	fAmpOut=GraphGetValueAt(ci,lpAmp->hGraph,pLogs[i])/4096.0;
					fAmpOut=pow(10.0,(fAmpOut*60.0-60.0)/20.0);
					pInts[i]=(short)(fAmpOut*16384.0+0.5);
					fAmpOut=GraphGetValueAt(ci,lpAmp->hGraph2,pLogs[i])/4096.0;
					fAmpOut=pow(10.0,(fAmpOut*60.0-60.0)/20.0);
					pInts[i+16390]=(short)(fAmpOut*16384.0+0.5);
					//pInts[i]=(short)GraphGetValueAt(ci,lpAmp->hGraph,(double)i/16386.0);
					//pInts[i+16390]=(short)GraphGetValueAt(ci,lpAmp->hGraph2,(double)i/16386.0);
				}
				//pInts[16384]=32765;
				//pInts[16383]=32765;		
				//pInts[16384+16390]=32765;
				//pInts[16383+16390]=32765;				
			}				
		}
	}
    
    ProgressDestroy(ci);
    
	GlobalUnlock(hLogs);
	GlobalFree(hLogs);

	GlobalUnlock(lpAmp->hRunTime);
	GlobalFree(lpAmp->hRunTime);
	lpAmp->hRunTime=NULL;
	
	GlobalUnlock(hBigMem);
	GlobalFree(hBigMem);
	hBigMem=0;
    
 	GlobalUnlock(hDistort);
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
// Fill the cool.ini file with our presets.  This is new as of CEPro1.1 and CE98
//////////////////////////////////////////////////////////////////////////////

_declspec(dllexport) BOOL FAR PASCAL XfmDefaultPresets(DWORD dwMode, FARPROC lpWriteIniString, LPSTR szIniFilename)
{
	if (dwMode==0)								// query version and group name.  
	{	lstrcpy(szIniFilename,TRANSFORMNAME2);	
		return 1;								// If new default presets are added, increment this return value
	}

	(*lpWriteIniString)(TRANSFORMNAME2,"Flat (no distortion),3,2,0,0,4096,4096,1,2,0,0,4096,4096,0,0",szIniFilename);
	(*lpWriteIniString)(TRANSFORMNAME2,
		"Super Loud!,3,4,0,0,2289,3979,3877,4096,4096,2547,1,2,0,0,4096,4096,1,1",szIniFilename);
	(*lpWriteIniString)(TRANSFORMNAME2,
		"Light Grunge,3,10,0,0,498,1981,1027,382,1448,2564,2149,1348,2460,3313,3052,2181,3301,4046,3753,3013,4096,4096,1,2,0,0,4096,4096,1,1",szIniFilename);
	(*lpWriteIniString)(TRANSFORMNAME2,
		"Maximum Jam!,3,9,0,0,311,2514,700,2281,1059,1781,1277,1814,1495,1631,2164,3230,2959,3929,4096,4096,0,5,0,0,763,3479,1759,3762,3893,2048,4096,4096,1,1",szIniFilename);
	(*lpWriteIniString)(TRANSFORMNAME2,
		"Sharp Fuzzzz,3,11,0,0,918,965,919,2164,1386,1415,1977,2081,1978,3180,2382,2530,2974,3047,2975,3862,3332,3446,4096,4096,0,11,0,0,420,499,421,1698,1012,1065,1588,1631,1589,2747,2040,2064,2663,2730,2664,3762,3114,3213,4096,4096,0,0",szIniFilename);
	(*lpWriteIniString)(TRANSFORMNAME2,
		"Distort Bass,3,6,0,0,1136,1298,1697,1048,2320,2680,3301,2980,4096,4096,0,6,0,0,1230,882,1619,1914,2709,2464,3254,3563,4096,4096,1,1",szIniFilename);
	(*lpWriteIniString)(TRANSFORMNAME2,
		"Mondo Messed Up,3,10,0,0,451,715,903,66,934,2863,1930,2897,1931,3646,2040,3230,2103,2880,3052,2797,4096,4096,0,12,0,0,93,732,545,1898,1137,1848,1386,2597,1945,2930,1946,666,2040,566,2195,3047,2196,2813,2881,2813,4096,3946,1,1",szIniFilename);
	(*lpWriteIniString)(TRANSFORMNAME2,
		"Tube Drive,3,4,0,0,573,2497,2711,2484,4096,2547,1,2,0,0,4096,4096,1,1",szIniFilename);
	(*lpWriteIniString)(TRANSFORMNAME2,
		"Cover and Distort,3,11,0,0,337,1347,978,872,1800,621,1801,1942,2486,1281,2487,2497,2868,3197,3263,3078,3454,3567,4096,4096,1,2,0,0,4096,4096,0,0",szIniFilename);

	return TRUE;
}

    
void DialogToStruct(COOLINFO *ci, HWND hWndDlg, DISTORT * lpAmp)
{	//char tmp[30];
	lpAmp->bSymmetric=IsDlgButtonChecked(hWndDlg,IDC_CHECK1)?TRUE:FALSE;
	lpAmp->iGraphViewing=(TabCtrl_GetCurSel(GetDlgItem(hWndDlg,IDC_TAB1)))?1:0;
	if (lpAmp->iGraphViewing)
	{	lpAmp->hGraph2=lpAmp->hGraphViewing;
		lpAmp->bSplines2=IsDlgButtonChecked(hWndDlg,IDC_CHECK2)?TRUE:FALSE;
	}
	else
	{	lpAmp->hGraph=lpAmp->hGraphViewing;
		lpAmp->bSplines1=IsDlgButtonChecked(hWndDlg,IDC_CHECK2)?TRUE:FALSE;
	}	
}

void StructToDialog(COOLINFO *ci, DISTORT *lpAmp, HWND hWndDlg)
{    //char tmp[30];                                               
	HDC hDC;
	TC_ITEM tie;
	char szItem[30];
	tie.mask=TCIF_TEXT;
	tie.pszText=szItem;
	
	CheckDlgButton(hWndDlg,IDC_CHECK1,lpAmp->bSymmetric);
	if (lpAmp->bSymmetric)
	{	lstrcpy(szItem,"(Symmetric)");
		TabCtrl_SetItem(GetDlgItem(hWndDlg,IDC_TAB1),0,&tie);
		lstrcpy(szItem,"(Symmetric)");
		TabCtrl_SetItem(GetDlgItem(hWndDlg,IDC_TAB1),1,&tie);				
		if (lpAmp->iGraphViewing==1)
		{	// Must switch into graph 0
			lpAmp->iGraphViewing=0;
			lpAmp->hGraphViewing=(lpAmp->iGraphViewing)?lpAmp->hGraph2:lpAmp->hGraph;
			hDC=GetDC(hWndDlg);
			GraphDraw(ci,lpAmp->hGraphViewing,hDC);
			ReleaseDC(hWndDlg,hDC);
		}		
	}
	else
	{	lstrcpy(szItem,"Positive");
		TabCtrl_SetItem(GetDlgItem(hWndDlg,IDC_TAB1),0,&tie);
		lstrcpy(szItem,"Negative");
		TabCtrl_SetItem(GetDlgItem(hWndDlg,IDC_TAB1),1,&tie);				
	}
	TabCtrl_SetCurSel(GetDlgItem(hWndDlg,IDC_TAB1),(lpAmp->iGraphViewing)?1:0);	
	lpAmp->hGraphViewing=(lpAmp->iGraphViewing)?lpAmp->hGraph2:lpAmp->hGraph;

	CheckDlgButton(hWndDlg,IDC_CHECK2,(lpAmp->iGraphViewing)?lpAmp->bSplines2:lpAmp->bSplines1);

	ShowWindow(GetDlgItem(hWndDlg,IDC_CHECK1),(lpAmp->iGraphViewing)?SW_HIDE:SW_SHOW);
	ShowWindow(GetDlgItem(hWndDlg,IDC_BUTTON1),(lpAmp->iGraphViewing)?SW_SHOW:SW_HIDE);					
	
	EnableWindow(GetDlgItem(hWndDlg,IDC_TAB1),!lpAmp->bSymmetric);

	GraphSetDblClkScales(ci,lpAmp->hGraph,60.0/4096,-60,100,60.0/4096.0,-60,100,(WORD)(GRAPH_HASRULERS|(lpAmp->bSplines1?GRAPH_SPLINES:0)));
	GraphSetDblClkNames(ci,lpAmp->hGraph,"Input Sample","dB","Output Sample","dB");
	GraphSetDblClkScales(ci,lpAmp->hGraph2,60.0/4096.0,-60,100,60.0/4096.0,-60,100,(WORD)(GRAPH_HASRULERS|(lpAmp->bSplines2?GRAPH_SPLINES:0)));
	GraphSetDblClkNames(ci,lpAmp->hGraph2,"Input Sample","dB","Output Sample","dB");
	
	hDC=GetDC(hWndDlg);
	GraphDraw(ci,lpAmp->hGraphViewing,hDC);
	ReleaseDC(hWndDlg,hDC);    
}    
    
typedef struct statinfo_t
{	HANDLE hThis;
	HANDLE hDistort;
	DISTORT * lpAmp;
	COOLINFO * ci;
} STATE;
    
__declspec(dllexport) BOOL FAR PASCAL DIALOGMsgProc(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam)
{   STATE * pState= (STATE *)GetWindowLong(hWndDlg,GWL_USERDATA);			

	if ((pState) && (pState->ci))
	{	if (DialogHook(pState->ci,hWndDlg,Message,wParam,lParam))
			return TRUE;
		if (GraphHook(pState->ci,pState->lpAmp->hGraphViewing,hWndDlg,Message,wParam,lParam))
			return TRUE;
	}

	switch(Message)
   	{	case WM_INITDIALOG:
		{	DISTORT * lpAmp = NULL;
			HANDLE hDistort = NULL;
			COOLINFO *ci=NULL;
			TC_ITEM tie;
			char szItem[30];
			REGISTERGRAPH rgraph;			
			REGISTERPRESETS rpinfo;
			HANDLE hState=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,sizeof(STATE));
			STATE * pState = (STATE *)GlobalLock(hState);
			pState->hThis=hState;
			
			
			ci=(COOLINFO *)lParam;
			hDistort=ci->hUserData;         
			lpAmp=(DISTORT *)GlobalLock(hDistort);
			pState->ci=ci;
			pState->lpAmp=lpAmp;
			pState->hDistort=hDistort;
			SetWindowLong(hWndDlg,GWL_USERDATA,(LONG)pState);
			
			ci->hWndDlg=hWndDlg;
			CenterDialog(ci,hWndDlg);

			rgraph.dwSize=sizeof(REGISTERGRAPH);
			rgraph.hWndDlg=hWndDlg;
			rgraph.uiNotify=ID_PREVIEWNOTIFY; 
			rgraph.uiRect=IDC_RECT;
			rgraph.uiDisplayText=IDC_DISPLAY;
			rgraph.hGraph=lpAmp->hGraph;
			RegisterGraph(ci, &rgraph);

			rgraph.dwSize=sizeof(REGISTERGRAPH);
			rgraph.hWndDlg=hWndDlg;
			rgraph.uiNotify=ID_PREVIEWNOTIFY; 
			rgraph.uiRect=IDC_RECT;
			rgraph.uiDisplayText=IDC_DISPLAY;
			rgraph.hGraph=lpAmp->hGraph2;
			RegisterGraph(ci, &rgraph);

	        lpAmp->iGraphViewing=0;
			
			tie.mask=TCIF_TEXT;
			tie.pszText=szItem;
			lstrcpy(szItem,"Positive");
			TabCtrl_InsertItem(GetDlgItem(hWndDlg,IDC_TAB1),0,&tie);
			lstrcpy(szItem,"Negative");
			TabCtrl_InsertItem(GetDlgItem(hWndDlg,IDC_TAB1),1,&tie);
			TabCtrl_SetCurSel(GetDlgItem(hWndDlg,IDC_TAB1),(lpAmp->iGraphViewing)?1:0);	
			lpAmp->hGraphViewing=(lpAmp->iGraphViewing)?lpAmp->hGraph2:lpAmp->hGraph;
	 
			FillMemory(&rpinfo,sizeof(REGISTERPRESETS),0);
			rpinfo.dwSize=sizeof(REGISTERPRESETS);
			lstrcpy(rpinfo.szGroupName,TRANSFORMNAME2);
			rpinfo.hWndDlg=hWndDlg;
			rpinfo.uiRect=ID_PRESETRECT;
			rpinfo.uiNotify=ID_PREVIEWNOTIFY;			
			RegisterPresets(ci, &rpinfo);

			StructToDialog(ci, lpAmp, hWndDlg);
	        //SetFocus(GetDlgItem(hWndDlg,IDOK));
					// using scripts, auto OK it.
			if (ci->cq->dwFlags&XF_AUTOOK)
		 		PostMessage(hWndDlg,WM_COMMAND,IDOK,0L);
	
		}
		break; 

		case WM_CLOSE:
			PostMessage(hWndDlg, WM_COMMAND, IDCANCEL, 0L);
			break;
     
		case WM_NOTIFY: 
		{   LPNMHDR phdr=(LPNMHDR)lParam;
			COOLINFO * ci = pState->ci;
			DISTORT * lpAmp = pState->lpAmp;
			switch (phdr->code) 
			{	case TCN_SELCHANGE: 
				{	short val;
					HDC hDC;
					val=(short)TabCtrl_GetCurSel(GetDlgItem(hWndDlg,IDC_TAB1));
					lpAmp->iGraphViewing=val;
					lpAmp->hGraphViewing=(lpAmp->iGraphViewing)?lpAmp->hGraph2:lpAmp->hGraph;
					ShowWindow(GetDlgItem(hWndDlg,IDC_CHECK1),(lpAmp->iGraphViewing)?SW_HIDE:SW_SHOW);
					ShowWindow(GetDlgItem(hWndDlg,IDC_BUTTON1),(lpAmp->iGraphViewing)?SW_SHOW:SW_HIDE);
					CheckDlgButton(hWndDlg,IDC_CHECK2,(lpAmp->iGraphViewing)?lpAmp->bSplines2:lpAmp->bSplines1);
					hDC=GetDC(hWndDlg);
					GraphDraw(ci,lpAmp->hGraphViewing,hDC);
					ReleaseDC(hWndDlg,hDC);    					
				} 
				break; 
			}     
		}
		break;

		case WM_PAINT:
		{	HDC hDC;
    		PAINTSTRUCT ps;
    		hDC=BeginPaint(hWndDlg,&ps);
			GraphDraw(pState->ci,pState->lpAmp->hGraphViewing,hDC);
			EndPaint(hWndDlg,&ps);
			return FALSE;
		}

		case WM_HELP:
			PostMessage(hWndDlg,WM_COMMAND,ID_HLP,0);
			break;
    
		case WM_COMMAND:
		{	COOLINFO * ci = pState->ci;
			DISTORT * lpAmp = pState->lpAmp;
			
			switch(LOWORD(wParam))
			{	case ID_HLP:
           			WinHelp(hWndDlg,ci->szHelpFile,HELP_KEY,(DWORD)((LPSTR)"Distort"));
					break;  
				case ID_PREVIEWNOTIFY:
					switch (HIWORD(wParam))
					{	case PRESETN_NOPRESETS:
							// Default presets are given using XfmDefaultPresets
							break;
						case PRESETN_CHOOSING:
							lpAmp->bSymmetric=TRUE;
							break;
						case PRESETN_CHOSEN:
							StructToDialog(ci, lpAmp, hWndDlg);
							break;
						case PRESETN_ADDING:
							DialogToStruct(ci, hWndDlg, lpAmp);
							break;
						case GRAPHN_MOUSEMOVE:
						case GRAPHN_BUTTONUP:
							break;
					}
					break;              	
              
				case IDC_BUTTON1:
				{	HDC hDC;
					GraphCopy(ci,lpAmp->hGraph2,lpAmp->hGraph);
					lpAmp->bSplines2=lpAmp->bSplines1;
					CheckDlgButton(hWndDlg,IDC_CHECK2,lpAmp->bSplines2);
					GraphSetDblClkScales(ci,lpAmp->hGraph2,60.0/4096.0,-60,100,60.0/4096.0,-60,100,(WORD)(GRAPH_HASRULERS|(lpAmp->bSplines2?GRAPH_SPLINES:0)));
					GraphSetDblClkNames(ci,lpAmp->hGraph2,"Input Sample","dB","Output Sample","dB");
					hDC=GetDC(hWndDlg);
					GraphDraw(ci,lpAmp->hGraphViewing,hDC);
					ReleaseDC(hWndDlg,hDC);    					
				}
				break;

				case IDC_CHECK2:
					if (lpAmp->iGraphViewing)
					{	lpAmp->bSplines2=IsDlgButtonChecked(hWndDlg,IDC_CHECK2)?TRUE:FALSE;
						GraphSetDblClkScales(ci,lpAmp->hGraph2,60.0/4096.0,-60,100,60.0/4096.0,-60,100,(WORD)(GRAPH_HASRULERS|(lpAmp->bSplines2?GRAPH_SPLINES:0)));
						GraphSetDblClkNames(ci,lpAmp->hGraph2,"Input Sample","dB","Output Sample","dB");			
					}
					else
					{	lpAmp->bSplines1=IsDlgButtonChecked(hWndDlg,IDC_CHECK2)?TRUE:FALSE;
						GraphSetDblClkScales(ci,lpAmp->hGraph,60.0/4096,-60,100,60.0/4096.0,-60,100,(WORD)(GRAPH_HASRULERS|(lpAmp->bSplines1?GRAPH_SPLINES:0)));
						GraphSetDblClkNames(ci,lpAmp->hGraph,"Input Sample","dB","Output Sample","dB");
					}
					break;

				case IDC_CHECK1:
				{	TC_ITEM tie;
					char szItem[30];
					tie.mask=TCIF_TEXT;
					tie.pszText=szItem;
					
					lpAmp->bSymmetric=IsDlgButtonChecked(hWndDlg,IDC_CHECK1);
					if (lpAmp->bSymmetric)
					{	lstrcpy(szItem,"(Symmetric)");
						TabCtrl_SetItem(GetDlgItem(hWndDlg,IDC_TAB1),0,&tie);
						lstrcpy(szItem,"(Symmetric)");
						TabCtrl_SetItem(GetDlgItem(hWndDlg,IDC_TAB1),1,&tie);				
					}
					else
					{	lstrcpy(szItem,"Positive");
						TabCtrl_SetItem(GetDlgItem(hWndDlg,IDC_TAB1),0,&tie);
						lstrcpy(szItem,"Negative");
						TabCtrl_SetItem(GetDlgItem(hWndDlg,IDC_TAB1),1,&tie);				
					}
					EnableWindow(GetDlgItem(hWndDlg,IDC_TAB1),!lpAmp->bSymmetric);
				}
				break;
              
				case IDCLOSE:
                case IDOK:
				{	if (GetFocus()==GetDlgItem(hWndDlg,ID_PRESETS))
                  {	PostMessage(hWndDlg,WM_COMMAND,ID_PRESETS,MAKELONG(GetDlgItem(hWndDlg,ID_PRESETS),LBN_DBLCLK));
                    break;
                  }
				  DialogToStruct(ci, hWndDlg,lpAmp);
				  
 				  lpAmp->iGraphViewing=0;
				  lpAmp->hGraphViewing=lpAmp->hGraph;
				  
             	  GlobalUnlock(pState->hDistort); 
             	  pState->hDistort=NULL;
                  EndDialog(hWndDlg, (LOWORD(wParam)==IDOK)?TRUE:2);
               }
            	 break;                                                    
              case IDCANCEL:
				GlobalUnlock(pState->hDistort);
                 pState->hDistort=NULL;
                 EndDialog(hWndDlg, FALSE);
                 break;
           }
		}
         break;   

		case WM_DESTROY:
		{	HANDLE h;
			STATE * pState= (STATE *)GetWindowLong(hWndDlg,GWL_USERDATA);
			h=pState->hThis;
			GlobalUnlock(h);
			GlobalFree(h);
			SetWindowLong(hWndDlg,GWL_USERDATA,0);
		}

    default:
		return FALSE;
   }
 
 	return TRUE;
}
