/////////////////////////////////////////////////////////////////////
// GENERAL PURPOSE DEFS FOR CREATING CUSTOM TRANSFORMATION EFFECTS //
/////////////////////////////////////////////////////////////////////

#ifndef __XFMS_H__
#define __XFMS_H__

#define XFM_MONO8		1
#define XFM_STEREO8 	2
#define XFM_MONO16		4
#define XFM_STEREO16	8
#define XFM_MONO32		16
#define XFM_STEREO32	32  

typedef DWORD (CALLBACK*      DWFARPROC)();
typedef void  (CALLBACK*      VFARPROC)();

typedef struct xfmquery_tag
{	char szName[80];  		// Appears in Menu (without elipsis...)
	char szCopyright[80];
	char szToolHelp[80];
	WORD wSupports;			// OR's of XFM_MONO16, XFM_STEREO16, XFM_MONO8, XFM_STEREO8
	DWORD dwFlags;			// Any of the XF_ flags OR'd together
							// Must have either XF_TRANSFORM or XF_GENERATE
							// May have XF_HASPRESETS, XF_HASSCRIPTS	
	DWORD dwUserDataLength;	// Length of transform's internal data
	char szStructDef[96];	// Array of chars representing types in user data area
	char szPresetDef[96];	// Array of chars 'y' to include in presets, 'n' to ignore.	
	WORD wExtra;			// Number of extra bytes to follow
} XFMQUERY;

#define XFM_VALIDLIBRARY 1157
#define XFM_VALIDLIBRARY98 1157
#define XFM_VALIDLIBRARY11 1157

#define XF_TRANSFORM    		0x0001	// This function transforms highlighted audio
#define XF_GENERATE				0x0002	// This function generates new pcm wave data
#define XF_USESPRESETSAPI		0x0004	// Has a presets box (and uses the presets/scripts API)
#define XF_USESGRAPHAPI			0x0008	// Uses the built in graph control functions
#define XF_MUSTHIGHLIGHT 		0x0010  // If set, function only enabled if a selection is highlighted (normal)
#define XF_MUSTHAVECLIP    	 	0x0020  // If set, function is grayed if nothing on clipboard
#define XF_MODIFIESTOENDOFVIEW  0x0040  // Set this if function modifies data outside user's given selection							
#define XF_MODIFIESTOENDOFFILE  0x0080	// Set this if function modifies data outside user's given selection							
#define XF_USESFFTAPI			0x0100	// Uses Cool's FFT functions
#define XF_NOSINGLEEDIT			0x0200	// Do not include if editing only one channel
#define XF_ANALYZE				0x0400	// This function analyzes data

#define XF_MODIFIESOUTSIDESEL	0x2000	// Set this if function modifies data outside selection, and fill dwModifyLeft and
										// dwModifyRight upon exiting XfmSetup
#define XF_SYSTEM				0x4000	// System function - does not show up in drop-down list
#define XF_PREVIEW				0x8000	// Preview API is being used

#define XF_USES32BITAPI			0x10000	// Uses lpReadEx and lpWriteEx and NOT lpReadData/lpWriteData
#define XF_AUTOCROSSFADE		0x20000 // Start and End (as defined by dwLoSample/dwHiSample) will be
										// crossfaded automatically roughly 200 samples (Uses 32-bit API)
#define XF_MULTIWAVEFORM		0x40000 // Set if this is an XFM that accepts more than one wave stream
#define XF_CONTIGUOUS			0x100000	// Set if read/write operations are contiguous, no re-reading, etc.
#define XF_CANEMBED				0x200000	// If dialog handles embedding (needs a dialog box template of child window type)
#define XF_AUTOOK				0x80000000L	// If set, wm_initdialog should send "OK" message to close
#define XF_COLORRED				0x40000000L	// If set, WM_CTLCOLOREDIT should color text red
#define XF_STRETCHES			0x20000000L // Set if result is of a different size and time markers
											// in that area should be proportionally stretched to match
#define XF_ALWAYSCALLSETUP		0x10000000L // even if "F3" repeat command without dialog, XfmSetup is called (with XF_AUTOOK set)

// Flags for new Read and Write functions lpReadEx and lpWriteEx
#define RWEX_MONO				0x01
#define RWEX_STEREO				0x02
#define RWEX_8BIT				0x08
#define RWEX_16BIT				0x10
#define RWEX_32BIT				0x20
#define RWEX_CLIP				0x1000
#define RWEX_DITHER				0x2000
#define RWEX_NOCROSSFADE		0x8000

// Old legacy COOLINFO members are indented

typedef struct coolinfo_tag
{	WORD wChannels;			// Number of channels
	WORD wBitsPerSample;	// Bit size, 8 or 16 for now
	WORD wBlockAlign;		// Bytes per sample (eg stereo 16-bit = 4)
	long lSamprate;			// Sample rate (8000,11025,22050, etc.) 
	HANDLE hUserData;		// Handle to specialized transform data, depends on DLL
	DWORD dwLoSample;       // Lowest sample to transform
	DWORD dwHiSample;		// Highest sample
	FARPROC lpTestFunction; 
	XFMQUERY FAR *cq;		// Pointer to query struct
	// General Purpose Tools
	FARPROC lpCenterDialog; // (HWND hWndDlg, int iUnused)  Center the dialog box
	
			// Reading and Writing audio data
			// OLD CODE - use lpReadEx, and lpWriteEx now.
			FARPROC lpReadData;		// (char huge *data,long offset,long amount)
			FARPROC lpWriteData;	// (char huge *data, DWORD offset,DWORD nbytes)	              
	
	// Progress Meter
	BOOL FAR *lpProgressCanceled;  // if TRUE, user hit Cancel button, you must stop processing NOW
	FARPROC lpProgressMeter; // (DWORD dwCurrent, DWORD dwTotal)  percent done is 100*dwCurrent/dwTotal
	FARPROC lpProgressCreate; // (LPCSTR szText)  szText is message to indicate type of processing
	FARPROC lpProgressDestroy; // (void)  Call to remove progress meter
	
	// Presets
	char far * szIniFile;    // Ini file
			FARPROC lpPresetsInit;	 // (HWND hWndDlg,LPSTR szGroupName)
			// OLD CODE -- ADDNEW will ask for name directly
			FARPROC lpHandleID_PRESETNAME;
			FARPROC lpHandleID_PRESETS;
			// OLD CODE -- ADDNEW will ask for name directly
			FARPROC lpHandleID_ADD;
			FARPROC lpHandleID_DEL;    
	
	// Graph control
	FARPROC lpGraphCreate;
	FARPROC lpGraphCount;
	FARPROC lpGraphGetPoint;
	FARPROC lpGraphGetValueAt;
	FARPROC lpGraphDraw;
	FARPROC lpGraphClear;
	FARPROC lpGraphInverse;
	FARPROC lpGraphDestroy;
			// OLD CODE -- call lpGraphControlHook in main msg proc for each graph
			FARPROC lpGraphHandleWM_LBUTTONDOWN;
			FARPROC lpGraphHandleWM_LBUTTONUP;
			FARPROC lpGraphHandleWM_LBUTTONDBLCLK;
			FARPROC lpGraphHandleWM_MOUSEMOVE;
	FARPROC lpGraphSetDblClkScales;
	FARPROC lpGraphSetDblClkNames; 
			FARPROC lpGraphSetDialog;	
	
			short iScriptFile;  
			short iScriptDialogStop; 
	
	HFONT hFont; 	// smaller font for dialogs
	
	// for XF_GENERATE type, must be filled out during XfmSetup
	BOOL	bReplacesHighlightedSelection;	// if TRUE, highlighted selection is deleted before inserting blanks
	DWORD	dwInsertBlankSamples;			// the number of samples to generate
	BOOL	bHasCoprocessor;					// if TRUE, use fp, otherwise use table lookups and other speedups
	
			// OLD CODE - should do windowing in XFM module itself, and call lpLFFT or lpConvolute	
			FARPROC lpFFT; // 	perform n-point FFT (radix-2 for now)
			FARPROC	lpWindowFFT;
			FARPROC lpIWindowFFT;
			FARPROC lpSetWindowType;  // 0..5: Triangular, von Hann, Hamming, Blackman, Cosine^2, Blackman-Harris
			FARPROC lpGetTempName; // get temporary file name
			
			FARPROC lpSetFFT16bit;
			FARPROC lpSetFFT8bit;
			FARPROC lpSetStereoFFT16bit;
			FARPROC lpSetStereoFFT8bit;
			FARPROC lpSetStereoFFT16bitInterleaved;
			FARPROC lpSetStereoFFT8bitInterleaved;
			FARPROC lpGetStereoFFT16bitInterleaved;
			FARPROC lpGetStereoFFT8bitInterleaved;    
	
	FARPROC lpCreateXfmVars;
	FARPROC lpSetXfmVar;
	DWFARPROC lpCallXfm;
	
			// OLD CODE - use lpDelayWriteEx, etc.
			// Delayed writing
			VFARPROC lpDelayWrite;
			VFARPROC lpDelayWriteInit;
			VFARPROC lpDelayWriteDestroy;
	
	HINSTANCE hInst;
	
	FARPROC lpGraphCopy;
	FARPROC lpCutData;
	FARPROC lpInsertBlankData;
	FARPROC lpDeleteXfmVars; 
	
	DWORD dwRightSample;    // Current viewing screen rightmost sample
	DWORD dwLeftSample;		// and leftmost sample  
	FARPROC lpGraphSetPoint;
	
	DWORD dwExtraFlags;
	FARPROC lpLFFT;
			DWFARPROC lpConvolute;
		
	DWORD dwModifyLeft;
	DWORD dwModifyRight;		 		
	
	// Used for Preview mode
	HINSTANCE hInstThisLibrary;
	DWFARPROC lpPreviewStart;
	DWFARPROC lpPreviewStop;
	DWFARPROC lpPreviewUpdate;

	DWFARPROC lpDisplayInit;

	DWORD dwTimeOffset;
	DWFARPROC lpLimits;
	WORD wInvalidEntry;
	HWND hWndDlg;

	DWFARPROC lpReadEx;
	DWFARPROC lpWriteEx;
	WORD wExtraReadFlags;
	WORD wExtraWriteFlags;
	DWORD dwFileID;
	DWFARPROC lpConvoluteEx;	
	DWFARPROC lpCutSamplesEx;
	DWFARPROC lpInsertSamplesEx;
	DWFARPROC lpGetMinMax;
	DWFARPROC lpDelayWriteInitEx;
	DWFARPROC lpDelayWriteEx;
	VFARPROC  lpDelayWriteDestroyEx;	
	char far * szHelpFile;    // Help file for main Cool Edit
	FARPROC lpProgressSetText; // (LPCSTR szText) to change progress text at any time
	FARPROC lpGraphControlHook;
	FARPROC lpDialogHook;
	int	iNumWaveforms;
	void * pNotUsed;
	FARPROC lpRegisterPresets;
	FARPROC lpRegisterGraph;
	FARPROC lpGraphSetProperty;
	DWFARPROC lpGetInfo;
	DWFARPROC lpGetIniString;
	DWFARPROC lpWriteIniString;
	DWFARPROC lpRegForms;
} COOLINFO;


#define CINFO_XFMNAME		1
#define CINFO_AMOVIECLSID	2
	 		

// Struct passed to RegisterGraph
typedef struct registergraph_tag
{	DWORD dwSize;
	HWND hWndDlg;
	UINT uiNotify;
	UINT uiRect;
	UINT uiDisplayText;
	WORD hGraph;
} REGISTERGRAPH;

#define GRAPHN_LBUTTONDOWN	201
#define GRAPHN_RBUTTONDOWN	202
#define GRAPHN_BUTTONUP		203
#define GRAPHN_MOUSEMOVE	204
#define GRAPHN_DBLCLK		205

#define GRAPH_LOGXAXIS		32		// Flags for GraphSetDblClkNames
#define GRAPH_SPLINES		64
#define GRAPH_HASRULERS		128

// Struct passed to RegisterPresets
typedef struct registerpresets_tag
{	DWORD dwSize;			// size of REGISTERPRESETS struct
	char szGroupName[64];
	HWND hWndDlg;
	UINT uiListBox;
	UINT uiAddButton;
	UINT uiDelButton;
	UINT uiNotify;
	UINT uiRect;
	char szPresetHeading[32];
} REGISTERPRESETS;

// Sent if using new presets format, RegisterPresets and PresetsInitEx
#define PRESETN_ADDING			101
#define PRESETN_ADDED			102
#define PRESETN_NOTADDED		110
#define PRESETN_DELETING		103
#define PRESETN_DELETED			104
#define PRESETN_NOTDELETED		111
#define PRESETN_CHOOSING		105
#define PRESETN_CHOSEN			106
#define PRESETN_NOTCHOSEN		112
#define PRESETN_NOPRESETS		107
#define PRESETN_INITIALIZING	108
#define PRESETN_INITIALIZED		109
#define PRESETREPLY_CANCEL		86

// Preview flags and such...
// Used in PreviewStart, just to see if the version of Cool supports Preview or not
#define PREVIEW_QUERY	16384

// Used in PreviewUpdate - if set, XfmDo is not called during update, but XfmUpdate IS called,
// otherwise XfmDo is called every time PreviewUpdate is called
#define PREVIEW_NORESTART 1

// Used in PreviewStart - if set, PREVIEW_NORESTART is ignored, plus XfmDo is called with data
// starting at the beginning of the highlight instead of data at the current location.  This flag
// makes sense only if PREVIEW_NORESTARTONLOOP is not specified in PreviewStart.
#define PREVIEW_RESTART_ALWAYS 4

// Used in PreviewStart - if set, XfmDo sees an infinite signal which is the looped version
// of the highlighted selection, otherwise XfmDo is called repeatedly for each loop
#define PREVIEW_NORESTARTONLOOP 2

// If 0 is passed to all Preview functions for the flags, XfmDo is called whenever Update is called,
// and every time the selection loops.  During update, read pointer is not reset to the start of 
// selection, but stays wherever it left off before the call to Update.
#define PREVIEW_BUFSIZE125		0x10
#define PREVIEW_BUFSIZE250		0x00
#define PREVIEW_BUFSIZE500		0x20
#define PREVIEW_BUFSIZE750		0x60
#define PREVIEW_BUFSIZE1000		0x30
#define PREVIEW_BUFSIZE1500		0x40
#define PREVIEW_BUFSIZE2000		0x50
#define PREVIEW_BUFSIZEMASK		0x70  // Do not use, it's the mask for the above sizes

#define PREVIEW_PRELOADSILENCE	0x80  // Load playback device with an initial 0.25 sec silence if threading problems exist

// Flags that are used in the HIWORD(wParam) of the callback command message
// To use preview, you must fill the ci->hWndDlg during WM_INITDIALOG
#define PREVN_STARTED 1	// Preview mode was started successfully
#define PREVN_STOPOK 2	// Preview mode ended normally
#define PREVN_STOPERR 3 // Preview mode ended because of error - XfmDo() returned non-zero
						// and lParam contains the error number.
#define PREVN_STOPNOW 4 // You must call PreviewStop in response to this message!!!

// Display Analysis Control
#define WM_SETDISPLAYDATA WM_USER+100
#define DISPLAY_ADDCHART 1
#define DISPLAY_BAKCOLOR 2
#define DISPLAY_LINECOLOR 3
#define DISPLAY_GRIDCOLOR 4
#define DISPLAY_TOPVALUE 5
#define DISPLAY_BOTTOMVALUE 6
#define DISPLAY_LEFTVALUE	7
#define DISPLAY_RIGHTVALUE	8
#define DISPLAY_TEXTCONTROL 9
#define DISPLAY_SAMPLERATE	10
#define DISPLAY_XLABEL		11
#define DISPLAY_YLABEL		12
#define DISPLAY_NORULERS	13
#define DISPLAY_CALLBACKTYPE  14

#define DISPLAY_MODE_DB		1	// Values to be used with DISPLAY_ADDCHART
#define DISPLAY_MODE_LOG	2
#define DISPLAY_MODE_WRAP	4
#define DISPLAY_MODE_NORMAL		8

#endif //__XFMS_H__
