/**************************************************************************
 *
 *  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
 *  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
 *  PURPOSE.
 *
 *  Copyright (c) 1992-1998 Syntrillium Software Corporation.  
 *	All Rights Reserved.
 * 
 **************************************************************************/
#include <windows.h>
#include <math.h>        

#include "xfms99dll.c" 
#include "resource.h"

// Usa an Internal Structure for your function, and access it through a handle
// It must be a packed structure, otherwise you must pad with char's or something
// so members start at locations evenly divisible by 8 or 16 depending on your
// compiler options

#pragma pack(1)
typedef struct dtmf_tag
{	double fVolume;
	double fShortSilence;
	double fLongSilence;
	double fToneOnTime;
	short iMode;
	char szDialString[256];
	char cPauseChar;
	short iCustom[8];
	double fTwist;
} DTMF;
#pragma pack()

// Structure Definition.  The user defined structure's elements must be
// described in the STRUCTDEFINITION string so Cool Edit can efficiently
// manage XFM modules, Presets, and Scripts.
//
// list variable types as:  c char
//							b BOOL
//							i short or WORD 
//							f float
//							l long or DWORD
//							d double
//							g WORD representing a graph id
//							h Global Handle  
//							s String of 256 chars
//
// Structure Preset Vars: 'y' to include in presets, 'n' to ignore.

#define STRUCTDEFINITION "ddddisciiiiiiiid"
#define PRESETDEFINITION "yyyyyyyyyyyyyyyy"

#define TRANSFORMNAME "SDK DTMF Signals"

#define MEMSIZE 64000
#define ID_NOTIFY 3000
#define CROSSFADETIME 100	// samples.  approx. 2 ms at 44.1KHz


#define dsetitem(Id,dVal) {ftos(tmp,dVal); SetDlgItemText(hWndDlg,Id,tmp);}

// So we don't have to include stdio.h, do our own flot to string conversion

void ftos(char *tmp, double d)
{	double dd;
	double mplier;
	short maxdecimal=9;
	
	if (d==0)
	{ *tmp='0';
	  tmp++;
	  *tmp=0x00;
	  return;
	} 
	
	if (d<0)
	{	*tmp='-';
		tmp++;
		d=-d;
	}
	d=d+(1.0/65535.0/32768.0);
	dd=d;         
	mplier=1.0;
	while (dd>=1.0)
	{	dd/=10.0;
		mplier*=10.0;	
	}   
	mplier/=10.0;             
	while (mplier>=1.0)
	{	*tmp='0'+(short)(d/mplier);
		tmp++;
		d=d-mplier*(short)(d/mplier);
		mplier/=10.0;	
	}
    if (d!=0.0)
    {	short digit;
    	*tmp='.';
    	tmp++;
    	while ((d!=0.0) && (maxdecimal!=0))
    	{	d*=10.0;
    		digit=(short)d;
    		d-=(double)digit;
    		*tmp='0'+digit;
    		tmp++;
    		maxdecimal--;    	    	
    	}             
    	while (*(tmp-1)=='0')
    		tmp--;
    	if (*(tmp-1)=='.')
    		tmp--;
    }
       
    *tmp=0x00;
}                                  

// This function is required for all DLLs

BOOL WINAPI DllMain (HANDLE hModule, DWORD fdwReason, LPVOID lpReserved)
{
   switch (fdwReason)
   {
      case DLL_PROCESS_ATTACH:
        /* Code from LibMain inserted here.  Return TRUE to keep the
            DLL loaded or return FALSE to fail loading the DLL.
 
            You may have to modify the code in your original LibMain to
            account for the fact that it may be called more than once.
            You will get one DLL_PROCESS_ATTACH for each process that
            loads the DLL. This is different from LibMain which gets
            called only once when the DLL is loaded. The only time this
            is critical is when you are using shared data sections.
            If you are using shared data sections for statically
            allocated data, you will need to be careful to initialize it
            only once. Check your code carefully.
 
            Certain one-time initializations may now need to be done for
            each process that attaches. You may also not need code from
            your original LibMain because the operating system may now
            be doing it for you.
         */
         break;
 
      case DLL_THREAD_ATTACH:
         /* Called each time a thread is created in a process that has
            already loaded (attached to) this DLL. Does not get called
            for each thread that exists in the process before it loaded
            the DLL.
 
            Do thread-specific initialization here.
         */
         break;
 
      case DLL_THREAD_DETACH:
         /* Same as above, but called when a thread in the process
            exits.
 
            Do thread-specific cleanup here.
         */
         break;
 
      case DLL_PROCESS_DETACH:
         /* Code from _WEP inserted here.  This code may (like the
            LibMain) not be necessary.  Check to make certain that the
            operating system is not doing it for you.
         */
         break;
   }
 
   /* The return value is only used for DLL_PROCESS_ATTACH; all other
      conditions are ignored.  */
   return TRUE;   // successful DLL_PROCESS_ATTACH
}

// Fill XFMQUERY structure with information regarding this file filter.
// If we only handle 32-bit floats, then Cool Edit will auto-convert
// to this format (or any format you ask for).  If optimizing especially
// for the 16-bit case, then OR to wSupports XFM_MONO16|XFM_STEREO16, 
// and check the ci->wBitsPerSample field for the native data format.
// Cool Edit will always format data in one of the types listed in
// the wSupports list.  You should include at least one MONO and one
// STEREO format, otherwise the module will only be active while working
// with Mono or Stereo data only, depending on the flag.
//
// Be sure to inlcude all the API sets that will be used in dwFlags,
// such as XF_USESPRESETSAPI, XF_USESGRAPHAPI, XF_USESFFTAPI, and 
// XF_USES32BITAPI.

__declspec(dllexport) short FAR PASCAL QueryXfm(XFMQUERY far * cq)
{   lstrcpy(cq->szName,TRANSFORMNAME);         
	lstrcpy(cq->szCopyright,"(c)1996-1999 Syntrillium Software Corporation");
	cq->wSupports=XFM_MONO32|XFM_STEREO32|0x8000;
	cq->dwFlags=XF_GENERATE|XF_USESPRESETSAPI|XF_USES32BITAPI;
	cq->dwUserDataLength = sizeof(DTMF);
	lstrcpy(cq->szStructDef,STRUCTDEFINITION);
	lstrcpy(cq->szPresetDef,PRESETDEFINITION);
	lstrcpy(cq->szToolHelp,"Generate DTMF or MF signals");
	return XFM_VALIDLIBRARY;
} 

__declspec(dllexport) BOOL FAR PASCAL XfmInit(COOLINFO far *ci)
{	DTMF FAR * lpAmp;
	HANDLE hDtmf;
	hDtmf=ci->hUserData;         
	lpAmp=(DTMF FAR *)GlobalLock(hDtmf);
	if (!lpAmp)
		return FALSE;
		
	// Initialize your structure variables here
	lpAmp->fVolume=0.5;
	lpAmp->fShortSilence=0.04;
	lpAmp->fLongSilence=0.5;
	lpAmp->fToneOnTime=0.1;
	lpAmp->iMode=0;
	lstrcpy(lpAmp->szDialString,"");
	lpAmp->cPauseChar=',';
	lpAmp->iCustom[0]=697;
	lpAmp->iCustom[1]=770;
	lpAmp->iCustom[2]=852;
	lpAmp->iCustom[3]=941;
	lpAmp->iCustom[4]=1209;
	lpAmp->iCustom[5]=1336;
	lpAmp->iCustom[6]=1477;
	lpAmp->iCustom[7]=1633;	
	lpAmp->fTwist=0;
	GlobalUnlock(hDtmf);
	return TRUE;
}

__declspec(dllexport) BOOL FAR PASCAL XfmDestroy(COOLINFO far *ci)
{   DTMF FAR * lpAmp;
	HANDLE hDtmf;
	hDtmf=ci->hUserData;         
	lpAmp=(DTMF FAR *)GlobalLock(hDtmf);
	if (!lpAmp)
		return FALSE;                       
		
 	// Do your de-initialization here (like freeing handles, etc.)
 	                
 	GlobalUnlock(hDtmf);
	return TRUE;
 }

__declspec(dllexport) BOOL FAR PASCAL XfmSetup(HWND hWnd, HINSTANCE hInst, COOLINFO far *ci)
{	short nRc;
	FARPROC lpfnDIALOGMsgProc;
	
	lpfnDIALOGMsgProc = GetProcAddress(hInst,(LPCSTR)MAKELONG(100,0));
	
	nRc = DialogBoxParam((HINSTANCE)hInst,(LPCSTR)MAKEINTRESOURCE(IDD_TRANSFORM), (HWND)hWnd, (DLGPROC)lpfnDIALOGMsgProc,(DWORD)ci);
	
	return nRc;
}

short tones0[16][2] =		// DTMF TONES
{	697,1209,  // 1
	697,1336,  // 2
	697,1477,  // 3
	770,1209,  // 4
	770,1336,  // 5
	770,1477,  // 6
	852,1209,  // 7
	852,1336,  // 8
	852,1477,  // 9
	941,1336,  // 0
	941,1209,  // *
	941,1477,  // #
	697,1633,  // a
	770,1633,  // b
	852,1633,  // c
	941,1633,  // d
};

short tones1[16][2] =		// MF TONES
{	700,900,
	700,1100,
	900,1100,
	700,1300,
	900,1300,
	1100,1300,
	700,1500,
	900,1500,
	1100,1500,
	1300,1500,
	1100,1700,
	1500,1700,
	900,1700,
	1300,1700,
	700,1700,
	0,0
};

short tones2[16][2] =		// CUSTOM TONES
{	697,1209,  // 1
	697,1336,  // 2
	697,1477,  // 3
	770,1209,  // 4
	770,1336,  // 5
	770,1477,  // 6
	852,1209,  // 7
	852,1336,  // 8
	852,1477,  // 9
	941,1336,  // 0
	941,1209,  // *
	941,1477,  // #
	697,1633,  // a
	770,1633,  // b
	852,1633,  // c
	941,1633,  // d
};

void BuildValidString(short iMode,unsigned char cPause,LPSTR szTmp)
{	lstrcpy(szTmp,"?");
	switch (iMode)
	{	case 2:
		case 0: 
			wsprintf(szTmp,"1234567890*#abcd%c",cPause);	
			break;
		case 1:
		    wsprintf(szTmp,"1234567890*#abc%c",cPause);		
	    	break;
	}
}

// This function is so we can return to Cool Edit the number
// of samples that will be generated given the user's settings.

DWORD CalculateSamples(COOLINFO far *ci,DTMF far *lpAmp)
{   short t;
	DWORD dwCount=0;
	BOOL bFirst=TRUE;
	char szValidString[20];
    
    BuildValidString(lpAmp->iMode,lpAmp->cPauseChar,(LPSTR)szValidString);
    
    for (t=0;t<lstrlen(lpAmp->szDialString);t++)
    {	char c;
    	short iOffset;
    	short w;
    	c=lpAmp->szDialString[t];
    	
    	iOffset=-1;
    	if (c==lpAmp->cPauseChar)
    	{	// Generate Silence
    	    dwCount+=(DWORD)(lpAmp->fLongSilence*(double)ci->lSamprate);    	    
    	    
    	}
    	else
    	{	for (w=0;w<lstrlen(szValidString);w++)
	    	{	if (szValidString[w]==c)
	    		{	iOffset=w;
	    			break;
	    		}
	    	}
	    	if (iOffset!=-1)
	    	{	if (!bFirst)
	    	    {
	    	     	dwCount+=(DWORD)(lpAmp->fShortSilence*(double)ci->lSamprate);
	    	    }
	    	    bFirst=FALSE;
	    	    
	    	    dwCount+=(DWORD)(lpAmp->fToneOnTime*(double)ci->lSamprate);
	    	}
	    	
        }    
    }
	dwCount+=(DWORD)(lpAmp->fShortSilence*(double)ci->lSamprate);
    
    return dwCount;
}
     			  
// All the real work happens here.  The main things to do here are:
// * Determine sample processing format (we do floats only here)
//   and set flags using various RWEX_ flags
// * Allocate our buffer for reading data into, modifying & writing
//   (or in our case, just writing only)
// * Start progress bar
// * Process all samples, calling ReadDataEx & WriteDataEx, and
//   call ProgressMeter periodically to update status bar
// * Destroy progress bar, de-allocate any mem, and that's it! 

__declspec(dllexport) DWORD FAR PASCAL XfmDo(COOLINFO far *ci)
{	DTMF FAR * lpAmp = NULL;
	HANDLE hDtmf = NULL;
	DWORD losamp, hisamp;
	DWORD nsamples=0L; 
	short t;
	char szValidString[20];
    HANDLE hBigMem;
    unsigned char far *pData;
    float *pfData;
    BOOL bFirst=TRUE; 
    DWORD startat;
	WORD wFlags=RWEX_32BIT;
	WORD wAlign;
	double fVolume1;
	double fVolume2;
	
	wFlags|=(WORD)((ci->wChannels==2)?RWEX_STEREO:RWEX_MONO);
	wAlign=sizeof(float)*ci->wChannels;
	
    hDtmf=ci->hUserData;         
	lpAmp=(DTMF FAR *)GlobalLock(hDtmf);
	losamp=ci->dwLoSample;
	hisamp=ci->dwHiSample;
	              	 
    hBigMem=GlobalAlloc(GMEM_MOVEABLE,4096*wAlign);
    pData=(unsigned char far *)GlobalLock(hBigMem);
    pfData=(float *)pData;
    startat=losamp;

	ProgressCreate(ci,"Generating DTMF signals",NULL);
    
    BuildValidString(lpAmp->iMode,lpAmp->cPauseChar,(LPSTR)szValidString);
	
	fVolume1=fVolume2=1.0;
	if (lpAmp->fTwist<0)
		fVolume2=pow(10.0,lpAmp->fTwist/20.0);
	else if (lpAmp->fTwist>0)
		fVolume1=pow(10.0,-lpAmp->fTwist/20.0);
	fVolume1*=lpAmp->fVolume;
	fVolume2*=lpAmp->fVolume;

	//lpAmp->fTwist  fVolume1

    if (lpAmp->iMode==2)
	{	// Copy over custom vals
		tones2[0][0]=tones2[1][0]=tones2[2][0]=tones2[12][0]=lpAmp->iCustom[0]; // 697
		tones2[3][0]=tones2[4][0]=tones2[5][0]=tones2[13][0]=lpAmp->iCustom[1]; // 770
		tones2[6][0]=tones2[7][0]=tones2[8][0]=tones2[14][0]=lpAmp->iCustom[2]; // 770
		tones2[9][0]=tones2[10][0]=tones2[11][0]=tones2[15][0]=lpAmp->iCustom[3]; // 770
		tones2[0][1]=tones2[3][1]=tones2[6][1]=tones2[10][1]=lpAmp->iCustom[4]; // 1209
		tones2[1][1]=tones2[4][1]=tones2[7][1]=tones2[9][1]=lpAmp->iCustom[5]; // 1336
		tones2[2][1]=tones2[5][1]=tones2[8][1]=tones2[11][1]=lpAmp->iCustom[6]; // 1477
		tones2[12][1]=tones2[13][1]=tones2[14][1]=tones2[15][1]=lpAmp->iCustom[7]; // 1633
	}
    
    for (t=0;t<lstrlen(lpAmp->szDialString);t++)
    {	char c;
    	short iOffset;
    	short w;
    	c=lpAmp->szDialString[t];
    	
    	iOffset=-1;
    	if (c==lpAmp->cPauseChar)
    	{	// Generate Silence
    	    DWORD dwSamples;
    	    dwSamples=(DWORD)(lpAmp->fLongSilence*(double)ci->lSamprate);
    	    while (dwSamples)
    	    {	DWORD dwAmount;     
    	    	
    	    	dwAmount=min(dwSamples,4096);
    	    	dwSamples-=dwAmount;
    	    	FillMemory(pData,dwAmount*wAlign,0);
    	    	WriteDataEx(ci,(char *)pData,startat,dwAmount,wFlags);
    	    	startat+=dwAmount; 
    	    	nsamples+=dwAmount;
    	    }    	    
    	}
    	else
    	{	
    	    
    		for (w=0;w<lstrlen(szValidString);w++)
	    	{	if (szValidString[w]==c)
	    		{	iOffset=w;
	    			break;
	    		}
	    	}
	    	if (iOffset!=-1)
	    	{	// Genrate Tones
	    	    short iTone1,iTone2;
	    	    DWORD dwSamples;
				DWORD dwSamplesDid;
    	    	double fToneInc1,fToneInc2;
	    	   	double fToneAt1,fToneAt2;
	    	   		
    	    	if (!bFirst)  // interval silence time
	    	    {   dwSamples=(DWORD)(lpAmp->fShortSilence*(double)ci->lSamprate);
		    	    while (dwSamples)
		    	    {	DWORD dwAmount;
		    	    	dwAmount=min(dwSamples,4096);
		    	    	dwSamples-=dwAmount;
		    	    	FillMemory(pData,dwAmount*wAlign,0);
		    	    	WriteDataEx(ci,(char *)pData,startat,dwAmount,wFlags);
		    	    	startat+=dwAmount; 
		    	    	nsamples+=dwAmount;
		    	    }	    	    
	    	    }
	    	    bFirst=FALSE;
	    	                              
	    	    if (lpAmp->iMode==0)
	    	    {	iTone1=tones0[iOffset][0];
	    	    	iTone2=tones0[iOffset][1];
	    	    }
	    	    else if (lpAmp->iMode==1)
	    	    {	iTone1=tones1[iOffset][0];
	    	    	iTone2=tones1[iOffset][1];
	    	    }
	    	    else if (lpAmp->iMode==2)
	    	    {	iTone1=tones2[iOffset][0];
	    	    	iTone2=tones2[iOffset][1];
	    	    }
	    	    
	    	    fToneAt1=fToneAt2=0;
	    	    fToneInc1=3.1415926535*2.0*(double)iTone1/(double)ci->lSamprate;
	    	    fToneInc2=3.1415926535*2.0*(double)iTone2/(double)ci->lSamprate;
	    	    
		    	dwSamples=(DWORD)(lpAmp->fToneOnTime*(double)ci->lSamprate);
				dwSamplesDid=0;

	    	    while (dwSamples)
	    	    {	DWORD dwAmount;
	    	    	DWORD t;
	    	    	double fVal;
					float *pf = pfData;
	    	    	
	    	    	dwAmount=min(dwSamples,4096);
	    	    	for (t=0;t<dwAmount;t++)
			    	{	fVal=16384.0*(fVolume1*sin(fToneAt1)+fVolume2*sin(fToneAt2));
			    	    if (dwSamples-t<CROSSFADETIME)
			    	    	fVal=fVal*(dwSamples-t)/(double)CROSSFADETIME;
						if (dwSamplesDid+t<CROSSFADETIME)
							fVal=fVal*t/(double)CROSSFADETIME;
			    	    *pf++=(float)fVal;
						if (ci->wChannels==2)
							*pf++=(float)fVal;
			    	    fToneAt1+=fToneInc1;
			    	    fToneAt2+=fToneInc2;
			    	}
			    	
	    	    	dwSamples-=dwAmount;
					dwSamplesDid+=dwAmount;
	    	    	
	    	    	WriteDataEx(ci,(char *)pData,startat,dwAmount,wFlags);
	    	    	startat+=dwAmount;
	    	    	nsamples+=dwAmount;
	    	    }	    	    
	    	}
        }    

    	ProgressMeter(ci,t,lstrlen(lpAmp->szDialString));
    	if (*ci->lpProgressCanceled) 
			break; 
    }
    
	{	DWORD dwSamples=(DWORD)(lpAmp->fShortSilence*(double)ci->lSamprate);
		while (dwSamples)
		{	DWORD dwAmount;
			dwAmount=min(dwSamples,4096);
			dwSamples-=dwAmount;
			FillMemory(pData,dwAmount*wAlign,0);
			WriteDataEx(ci,(char *)pData,startat,dwAmount,wFlags);
			startat+=dwAmount; 
			nsamples+=dwAmount;
		}
	}

    ProgressDestroy(ci);

	GlobalUnlock(hBigMem);
	GlobalFree(hBigMem);
	
    GlobalUnlock(hDtmf);
	return nsamples;
}
		

    
// This function is called all over to copy contents of dialog box
// to our structure

#define ITEMS_ALL			0xffff
#define ITEMS_EDITS1TO8		0x0001
#define ITEMS_DIALSTRING	0x0002
#define ITEMS_TONETIME		0x0004
#define ITEMS_BREAKTIME		0x0008
#define ITEMS_PAUSETIME		0x0010
#define ITEMS_AMPLITUDE		0x0020
#define ITEMS_PAUSECHAR		0x0040
#define ITEMS_RADIOS		0x0080

    
void DialogToStruct(COOLINFO FAR *ci, HWND hWndDlg, DTMF FAR * lpAmp, WORD wWhichItems)
{	double fNewVal;

	if (wWhichItems&ITEMS_AMPLITUDE)
	{	// To keep text edit fields within bounds, use the Limits function to get
		// the values from text edit boxes.		
		Limits(ci, hWndDlg,IDC_AMPLITUDE, 0,200,&fNewVal);
		lpAmp->fVolume=fNewVal/100.0;
		Limits(ci, hWndDlg,IDC_AMPLITUDE2, -100,100,&fNewVal);
		lpAmp->fTwist=fNewVal;
	}

	if (wWhichItems&ITEMS_BREAKTIME)
	{	Limits(ci, hWndDlg,IDC_BREAKTIME, 0,10000,&fNewVal);
		lpAmp->fShortSilence=fNewVal/1000.0;
	}

	if (wWhichItems&ITEMS_PAUSETIME)
	{	Limits(ci, hWndDlg,IDC_PAUSETIME, 1,10000,&fNewVal);
		lpAmp->fLongSilence=fNewVal/1000.0;
	}

	if (wWhichItems&ITEMS_TONETIME)
	{	Limits(ci, hWndDlg,IDC_TONETIME, 1,10000,&fNewVal);
		lpAmp->fToneOnTime=fNewVal/1000.0;
	}

	if (wWhichItems&ITEMS_EDITS1TO8)
	{	Limits(ci, hWndDlg,IDC_EDIT1,0,50000,&fNewVal);
		lpAmp->iCustom[0]=(short)fNewVal;
		Limits(ci, hWndDlg,IDC_EDIT2,0,50000,&fNewVal);
		lpAmp->iCustom[1]=(short)fNewVal;
		Limits(ci, hWndDlg,IDC_EDIT3,0,50000,&fNewVal);
		lpAmp->iCustom[2]=(short)fNewVal;
		Limits(ci, hWndDlg,IDC_EDIT4,0,50000,&fNewVal);
		lpAmp->iCustom[3]=(short)fNewVal;
		Limits(ci, hWndDlg,IDC_EDIT5,0,50000,&fNewVal);
		lpAmp->iCustom[4]=(short)fNewVal;
		Limits(ci, hWndDlg,IDC_EDIT6,0,50000,&fNewVal);
		lpAmp->iCustom[5]=(short)fNewVal;
		Limits(ci, hWndDlg,IDC_EDIT7,0,50000,&fNewVal);
		lpAmp->iCustom[6]=(short)fNewVal;
		Limits(ci, hWndDlg,IDC_EDIT8,0,50000,&fNewVal);
		lpAmp->iCustom[7]=(short)fNewVal;
	}

	if (wWhichItems&ITEMS_RADIOS)
	{	if (IsDlgButtonChecked(hWndDlg,IDC_RADIO1))
			lpAmp->iMode=0;
		else if (IsDlgButtonChecked(hWndDlg,IDC_RADIO2))
			lpAmp->iMode=1;
		else if (IsDlgButtonChecked(hWndDlg,IDC_RADIO3))
			lpAmp->iMode=2;
	}
	
	if (wWhichItems&ITEMS_DIALSTRING)
		GetDlgItemText(hWndDlg,IDC_DIALSTRING,lpAmp->szDialString,255);
	
	if (wWhichItems&ITEMS_PAUSECHAR)
	{	char tmp[6];
		GetDlgItemText(hWndDlg,IDC_PAUSECHAR,tmp,5);
		lpAmp->cPauseChar=tmp[0];
		if (lpAmp->cPauseChar<32)
			lpAmp->cPauseChar=' ';	            	
	}
	
}

// This function is called all over to copy contents of our
// structure to the dialog box.

void StructToDialog(COOLINFO FAR *ci, DTMF FAR *lpAmp, HWND hWndDlg)
{   char tmp[30];
    BOOL bEnabled=FALSE;

    dsetitem(IDC_AMPLITUDE,lpAmp->fVolume*100.0);
    dsetitem(IDC_BREAKTIME,lpAmp->fShortSilence*1000.0);
    dsetitem(IDC_PAUSETIME,lpAmp->fLongSilence*1000.0);
    dsetitem(IDC_TONETIME,lpAmp->fToneOnTime*1000.0);

    CheckDlgButton(hWndDlg,IDC_RADIO1,(lpAmp->iMode==0)?TRUE:FALSE);
    CheckDlgButton(hWndDlg,IDC_RADIO2,(lpAmp->iMode==1)?TRUE:FALSE);
    CheckDlgButton(hWndDlg,IDC_RADIO3,(lpAmp->iMode==2)?TRUE:FALSE);
    SetDlgItemText(hWndDlg,IDC_DIALSTRING,lpAmp->szDialString);
    wsprintf(tmp,"%c",lpAmp->cPauseChar);
    SetDlgItemText(hWndDlg,IDC_PAUSECHAR,tmp);
    BuildValidString(lpAmp->iMode,lpAmp->cPauseChar,(LPSTR)tmp);
    SetDlgItemText(hWndDlg,IDC_USABLECHARS,tmp);
	if (lpAmp->iMode==2)
		bEnabled=TRUE;
	EnableWindow(GetDlgItem(hWndDlg,IDC_BUTTON1),bEnabled);
	EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT1),bEnabled);
	EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT2),bEnabled);
	EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT3),bEnabled);
	EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT4),bEnabled);
	EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT5),bEnabled);
	EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT6),bEnabled);
	EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT7),bEnabled);
	EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT8),bEnabled);

	dsetitem(IDC_EDIT1,(double)lpAmp->iCustom[0]);
	dsetitem(IDC_EDIT2,(double)lpAmp->iCustom[1]);
	dsetitem(IDC_EDIT3,(double)lpAmp->iCustom[2]);
	dsetitem(IDC_EDIT4,(double)lpAmp->iCustom[3]);
	dsetitem(IDC_EDIT5,(double)lpAmp->iCustom[4]);
	dsetitem(IDC_EDIT6,(double)lpAmp->iCustom[5]);
	dsetitem(IDC_EDIT7,(double)lpAmp->iCustom[6]);
	dsetitem(IDC_EDIT8,(double)lpAmp->iCustom[7]);
	dsetitem(IDC_AMPLITUDE2,lpAmp->fTwist);
}    

_declspec(dllexport) BOOL FAR PASCAL XfmDefaultPresets(DWORD dwMode, FARPROC lpWriteIniString, LPSTR szIniFilename)
{
	if (dwMode<1) // query version and group name
	{	lstrcpy(szIniFilename,TRANSFORMNAME);
		return 1;
	}
	
	(*lpWriteIniString)(TRANSFORMNAME,"Syntrillium Software,3,0.5,0.08,0.4,0.15,0,1480-941-4327,44,697,770,852,941,1,1,1,1,0",szIniFilename);
	
	return TRUE;
}


// To keep track of stuff local to our dialog box instance, we keep a pointer
// to this state info in the extra data portion using SetWindowLong

typedef struct statinfo_t
{	HANDLE hThis;
	HANDLE hDtmf;
	DTMF * lpAmp;
	COOLINFO * ci;
} STATE;

__declspec(dllexport) BOOL FAR PASCAL DIALOGMsgProc(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam)
{   STATE * pState= (STATE *)GetWindowLong(hWndDlg,GWL_USERDATA);			

	// If using Presets, we need to call DialogHook so Cool Edit can handle stuff
	// (similiarly, if using graphs, call GraphHook as well)
	if ((pState) && (pState->ci) && (DialogHook(pState->ci,hWndDlg,Message,wParam,lParam)))
		return TRUE;

	switch(Message)
   	{	case WM_INITDIALOG:
		{	DTMF FAR * lpAmp = NULL;
			HANDLE hDtmf = NULL;
			COOLINFO far *ci=NULL;
			REGISTERPRESETS rpinfo;
			HANDLE hState=GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT,sizeof(STATE));
			STATE * pState = (STATE *)GlobalLock(hState);
			
			pState->hThis=hState;
			
			ci=(COOLINFO far *)lParam;
			hDtmf=ci->hUserData;         
			lpAmp=(DTMF FAR *)GlobalLock(hDtmf);
			pState->ci=ci;
			pState->lpAmp=lpAmp;
			pState->hDtmf=hDtmf;
			SetWindowLong(hWndDlg,GWL_USERDATA,(LONG)pState);
			ci->hWndDlg=hWndDlg;		// Must tell Cool Edit who we are!
			CenterDialog(ci,hWndDlg);
	        
			
			FillMemory(&rpinfo,sizeof(REGISTERPRESETS),0);
			rpinfo.dwSize=sizeof(REGISTERPRESETS);
			lstrcpy(rpinfo.szGroupName,TRANSFORMNAME);
			rpinfo.hWndDlg=hWndDlg;
			rpinfo.uiRect=ID_PRESETSRECT;
			rpinfo.uiNotify=ID_NOTIFY;			
			RegisterPresets(ci, &rpinfo);

			// Do any other initialization here, like filling list boxes, etc.

			StructToDialog(ci, lpAmp, hWndDlg);

	 		// Auto OK it if necessary
	 		if (ci->cq->dwFlags&XF_AUTOOK)
		 		PostMessage(hWndDlg,WM_COMMAND,IDOK,0L);	        
		}    		
        break;

		// For verification of text edit fields, turns text RED if invalid
		case WM_CTLCOLOREDIT:
		{	HDC hDC=(HDC)wParam;
			HWND hWndEdit=(HWND)lParam;
		
			if ((pState->ci->wInvalidEntry) && (GetDlgItem(hWndDlg,pState->ci->wInvalidEntry)==hWndEdit))
			{	// Color control red
				SetTextColor(hDC,RGB(255,0,0));
				return (BOOL)GetStockObject(WHITE_BRUSH);
			}
			else // Color text black
				return FALSE;		
		}
		break;

		case WM_CLOSE:
			PostMessage(hWndDlg, WM_COMMAND, IDCANCEL, 0L);
			break; 
	 
		case WM_HELP:
			PostMessage(hWndDlg,WM_COMMAND,ID_HLP,0);
			break;

		case WM_COMMAND:
		{	COOLINFO * ci = pState->ci;
			DTMF * lpAmp = pState->lpAmp;
			double fNewVal;	// for limits testing
					
			switch(LOWORD(wParam))
			{	case ID_HLP:
           			WinHelp(hWndDlg,pState->ci->szHelpFile,HELP_KEY,(DWORD)((LPSTR)"DTMF Tones"));
					break;  

				case ID_NOTIFY:
					switch (HIWORD(wParam))
					{	case PRESETN_NOPRESETS:
							break;
						case PRESETN_ADDING:
							DialogToStruct(ci, hWndDlg, lpAmp, ITEMS_ALL);
							break;
						case PRESETN_CHOOSING:
							lpAmp->fTwist=0;
							break;
						case PRESETN_CHOSEN:
							 StructToDialog(ci, lpAmp, hWndDlg);
				 			break;
					}
					break;  
				
				case IDC_BUTTON1:
				{	char tmp[30];
					dsetitem(IDC_EDIT1,697);
					dsetitem(IDC_EDIT2,770);
					dsetitem(IDC_EDIT3,852);
					dsetitem(IDC_EDIT4,941);
					dsetitem(IDC_EDIT5,1209);
					dsetitem(IDC_EDIT6,1336);
					dsetitem(IDC_EDIT7,1477);
					dsetitem(IDC_EDIT8,1633);
				}
				break;

				case IDC_EDIT1:
					Limits(ci, hWndDlg,wParam,0,50000,&fNewVal);  // Remember to use wParam, not LOWORD(wParam)!
					lpAmp->iCustom[0]=(short)fNewVal;
					break;					
				case IDC_EDIT2:
					Limits(ci, hWndDlg,wParam,0,50000,&fNewVal);
					lpAmp->iCustom[1]=(short)fNewVal;
					break;
				case IDC_EDIT3:
					Limits(ci, hWndDlg,wParam,0,50000,&fNewVal);
					lpAmp->iCustom[2]=(short)fNewVal;
					break;
				case IDC_EDIT4:
					Limits(ci, hWndDlg,wParam,0,50000,&fNewVal);
					lpAmp->iCustom[3]=(short)fNewVal;
					break;
				case IDC_EDIT5:
					Limits(ci, hWndDlg,wParam,0,50000,&fNewVal);
					lpAmp->iCustom[4]=(short)fNewVal;
					break;
				case IDC_EDIT6:
					Limits(ci, hWndDlg,wParam,0,50000,&fNewVal);
					lpAmp->iCustom[5]=(short)fNewVal;
					break;
				case IDC_EDIT7:
					Limits(ci, hWndDlg,wParam,0,50000,&fNewVal);
					lpAmp->iCustom[6]=(short)fNewVal;
					break;
				case IDC_EDIT8:
					Limits(ci, hWndDlg,wParam,0,50000,&fNewVal);
					lpAmp->iCustom[7]=(short)fNewVal;
					break;

				case IDC_TONETIME:				  				  
					Limits(ci, hWndDlg,wParam, 1,10000,&fNewVal);
					lpAmp->fToneOnTime=fNewVal/1000.0;
					break;

				case IDC_BREAKTIME:
					Limits(ci, hWndDlg,wParam, 0,10000,&fNewVal);
					lpAmp->fShortSilence=fNewVal/1000.0;					
					break;

				case IDC_PAUSETIME:
					Limits(ci, hWndDlg,wParam, 1,10000,&fNewVal);
					lpAmp->fLongSilence=fNewVal/1000.0;
					break;

				case IDC_AMPLITUDE:
					Limits(ci, hWndDlg,wParam, 0,200,&fNewVal);	
					lpAmp->fVolume=fNewVal/100.0;
					break;

				case IDC_AMPLITUDE2:
					Limits(ci, hWndDlg,wParam, -100,100,&fNewVal);	
					lpAmp->fTwist=fNewVal;
					break;

				case IDC_RADIO1:
				case IDC_RADIO2:
				case IDC_RADIO3:
				{	char tmp[29];
					BOOL bEnabled=FALSE;
					int iMode=0;
                	if (IsDlgButtonChecked(hWndDlg,IDC_RADIO2))
						iMode=1;
					else if (IsDlgButtonChecked(hWndDlg,IDC_RADIO3))
						iMode=2;
	            	BuildValidString((short)iMode,lpAmp->cPauseChar,(LPSTR)tmp);
					//SetEditWindows(hWndDlg,lpAmp->iMode);
    
    				SetDlgItemText(hWndDlg,IDC_USABLECHARS,tmp);
					if ((LOWORD(wParam)==IDC_RADIO3))
						bEnabled=TRUE;
					EnableWindow(GetDlgItem(hWndDlg,IDC_BUTTON1),bEnabled);
					EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT1),bEnabled);
					EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT2),bEnabled);
					EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT3),bEnabled);
					EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT4),bEnabled);
					EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT5),bEnabled);
					EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT6),bEnabled);
					EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT7),bEnabled);
					EnableWindow(GetDlgItem(hWndDlg,IDC_EDIT8),bEnabled);

				}
              	break;	

				case IDCLOSE:
           		case IDOK:
				{	
              		DialogToStruct(ci, hWndDlg,lpAmp,ITEMS_ALL);
                  
					GlobalUnlock(pState->hDtmf); 
             		pState->hDtmf=NULL;
             	  
             		ci->bReplacesHighlightedSelection=TRUE;
     				ci->dwInsertBlankSamples=CalculateSamples(ci,lpAmp);
     			  
					// Alwyas return a 1 or TRUE for OK, and a 2 for CLOSE
					EndDialog(hWndDlg, (LOWORD(wParam)==IDOK)?TRUE:2);
				}
				break;                                                    

				case IDCANCEL:
					GlobalUnlock(pState->hDtmf);
					pState->hDtmf=NULL;
					
					// Always return a 0 for CANCEL
					EndDialog(hWndDlg, FALSE);
					break;
			}
		}
		break;

		case WM_DESTROY:
		{	HANDLE h;	// Free our state info struct
			STATE * pState= (STATE *)GetWindowLong(hWndDlg,GWL_USERDATA);
			h=pState->hThis;
			GlobalUnlock(h);
			GlobalFree(h);
			SetWindowLong(hWndDlg,GWL_USERDATA,0);
		}

		default:
			return FALSE;
	}
 	return TRUE;
}
